/*
 * Wazuh Vulnerability scanner
 * Copyright (C) 2015, Wazuh Inc.
 * April 5, 2024.
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation.
 */

#ifndef _WDB_DATA_EXCEPTION_HPP
#define _WDB_DATA_EXCEPTION_HPP

#include <stdexcept>

/**
 * @brief WdbDataException class.
 *
 */
class WdbDataException : public std::exception
{
public:
    /**
     * @brief Overload what() method.
     *
     * @return const char*
     */
    // LCOV_EXCL_START
    const char* what() const noexcept override
    {
        return m_msg.what();
    }

    /**
     * @brief Get agent id.
     *
     * @return const std::string&
     */
    const std::string& agentId() const
    {
        return m_agentId;
    }
    // LCOV_EXCL_STOP

    /**
     * @brief Construct a new Os Data Exception object
     *
     * @param message
     * @param agentId
     */
    explicit WdbDataException(const std::string& message, std::string_view agentId)
        : m_msg {message} // NOLINT
        , m_agentId {agentId}
    {
    }

private:
    std::runtime_error m_msg;
    std::string m_agentId;
};

#endif // _WDB_DATA_EXCEPTION_HPP
