/*
 * Wazuh databaseFeedManager
 * Copyright (C) 2015, Wazuh Inc.
 * January 4, 2024.
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation.
 */
#ifndef _MOCK_OSDATACACHE_HPP
#define _MOCK_OSDATACACHE_HPP

#include "gmock/gmock.h"
#include "gtest/gtest.h"

#include "osDataCache.hpp"
#include "singleton.hpp"

#include "json.hpp"

/**
 * @class MockOsDataCache
 *
 * @brief Mock class for simulating a database feed manager object.
 *
 * The `MockOsDataCache` class is designed to simulate the behavior of an
 * os data cache for testing purposes. It provides mock implementations of methods and
 * allows you to set expectations on method calls and their return values for testing.
 *
 * This class is used in unit tests only to verify interactions with a content
 * register without actually performing real operations on it.
 */
class MockOsDataCache : public Singleton<MockOsDataCache>
{
public:
    MockOsDataCache() = default;

    virtual ~MockOsDataCache() = default;

    /**
     * @brief Mock method for getOsData.
     *
     * @note This method is intended for testing purposes and does not perform any real action.
     */
    MOCK_METHOD(Os, getOsData, (const std::string& agentId), ());

    /**
     * @brief Mock method for setOsData.
     *
     * @note This method is intended for testing purposes and does not perform any real action.
     */
    MOCK_METHOD(void, setOsData, (const std::string& agentId, const Os& osData), ());
};

#endif // _MOCK_OSDATACACHE_HPP
